@@ -1,4 +1,3 @@
-{-# LANGUAGE CPP #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE BangPatterns #-}
 
@@ -11,9 +10,9 @@ module Network.DNS.Resolver (
   , resolvInfo
   , resolvTimeout
   , resolvRetry
-  , resolvEDNS
   , resolvConcurrent
   , resolvCache
+  , resolvQueryControls
   -- ** Specifying DNS servers
   , FileOrNumericHost(..)
   -- ** Configuring cache
@@ -30,16 +29,6 @@ module Network.DNS.Resolver (
   , withResolvers
   ) where
 
-#if !defined(mingw32_HOST_OS)
-#define POSIX
-#else
-#define WIN
-#endif
-
-#if __GLASGOW_HASKELL__ < 709
-#define GHC708
-#endif
-
 import Control.Exception as E
 import qualified Crypto.Random as C
 import qualified Data.ByteString as BS
@@ -47,21 +36,14 @@ import Data.IORef (IORef)
 import qualified Data.IORef as I
 import qualified Data.List.NonEmpty as NE
 import Network.Socket (AddrInfoFlag(..), AddrInfo(..), PortNumber, HostName, SocketType(Datagram), getAddrInfo, defaultHints)
-import Prelude hiding (lookup)
-
-#if defined(WIN)
-import qualified Data.List.Split as Split
-import Foreign.C.String
-import Foreign.Marshal.Alloc (allocaBytes)
-#else
-import Data.Char (isSpace)
-#endif
+import Prelude
 
 import Network.DNS.Imports
 import Network.DNS.Memo
+import Network.DNS.Resolver.Internal
 import Network.DNS.Transport
-import Network.DNS.Types
 import Network.DNS.Types.Internal
+import Network.DNS.Types.Resolver
 
 ----------------------------------------------------------------
 
@@ -83,27 +65,6 @@ makeResolvSeed conf = ResolvSeed conf <$> findAddresses
     mkAddrs []     = E.throwIO BadConfiguration
     mkAddrs (l:ls) = (:|) <$> makeAddrInfo l Nothing <*> forM ls (`makeAddrInfo` Nothing)
 
-getDefaultDnsServers :: FilePath -> IO [String]
-#if defined(WIN)
-foreign import ccall "getWindowsDefDnsServers" getWindowsDefDnsServers :: CString -> Int -> IO Word32
-getDefaultDnsServers _ = do
-  allocaBytes 128 $ \cString -> do
-     res <- getWindowsDefDnsServers cString 128
-     case res of
-       0 -> do
-         addresses <- peekCString cString
-         return $ filter (not . null) . Split.splitOn "," $ addresses
-       _ -> do
-         -- TODO: Do proper error handling here.
-         return mempty
-#else
-getDefaultDnsServers file = toAddresses <$> readFile file
-  where
-    toAddresses :: String -> [String]
-    toAddresses cs = map extract (filter ("nameserver" `isPrefixOf`) (lines cs))
-    extract = reverse . dropWhile isSpace . reverse . dropWhile isSpace . drop 11
-#endif
-
 makeAddrInfo :: HostName -> Maybe PortNumber -> IO AddrInfo
 makeAddrInfo addr mport = do
     let flgs = [AI_ADDRCONFIG, AI_NUMERICHOST, AI_PASSIVE]
