@@ -16,35 +16,49 @@ import System.Timeout (timeout)
 
 import Network.DNS.IO
 import Network.DNS.Imports
-import Network.DNS.Types
 import Network.DNS.Types.Internal
+import Network.DNS.Types.Resolver
 
 -- | Check response for a matching identifier and question.  If we ever do
 -- pipelined TCP, we'll need to handle out of order responses.  See:
 -- https://tools.ietf.org/html/rfc7766#section-7
-checkResp :: [Question] -> Identifier -> DNSMessage -> Bool
-checkResp q seqno resp =
-   (identifier (header resp) == seqno) && (q == (question resp))
+--
+checkResp :: Question -> Identifier -> DNSMessage -> Bool
+checkResp q seqno = isNothing . checkRespM q seqno
+
+-- When the response 'RCODE' is 'FormatErr', the server did not understand our
+-- query packet, and so is not expected to return a matching question.
+--
+checkRespM :: Question -> Identifier -> DNSMessage -> Maybe DNSError
+checkRespM q seqno resp
+  | identifier (header resp) /= seqno = Just SequenceNumberMismatch
+  | FormatErr <- rcode $ flags $ header resp
+  , []        <- question resp        = Nothing
+  | [q] /= question resp              = Just QuestionMismatch
+  | otherwise                         = Nothing
 
 ----------------------------------------------------------------
 
 data TCPFallback = TCPFallback deriving (Show, Typeable)
 instance Exception TCPFallback
 
-type Rslv0 = Bool -> (Socket -> IO DNSMessage)
+type Rslv0 = QueryControls -> (Socket -> IO DNSMessage)
            -> IO (Either DNSError DNSMessage)
 
-type Rslv1 = [Question]
-          -> [ResourceRecord]
+type Rslv1 = Question
           -> Int -- Timeout
           -> Int -- Retry
           -> Rslv0
 
-type TcpRslv = Identifier -> AddrInfo -> [Question] -> Int -- Timeout
-            -> Bool -> IO DNSMessage
+type TcpRslv = AddrInfo
+            -> Question
+            -> Int -- Timeout
+            -> QueryControls
+            -> IO DNSMessage
 
-type UdpRslv = [ResourceRecord] -> Int -- Retry
-            -> (Socket -> IO DNSMessage) -> TcpRslv
+type UdpRslv = Int -- Retry
+            -> (Socket -> IO DNSMessage)
+            -> TcpRslv
 
 -- In lookup loop, we try UDP until we get a response.  If the response
 -- is truncated, we try TCP once, with no further UDP retries.
@@ -60,65 +74,75 @@ type UdpRslv = [ResourceRecord] -> Int -- Retry
 --
 -- Future improvements might also include support for TCP on the
 -- initial query.
+--
+-- This function merges the query flag overrides from the resolver
+-- configuration with any additional overrides from the caller.
+--
 resolve :: Domain -> TYPE -> Resolver -> Rslv0
-resolve dom typ rlv ad rcv
+resolve dom typ rlv qctls rcv
   | isIllegal dom = return $ Left IllegalDomain
-  | onlyOne       = resolveOne        (head nss) (head gens) q edns tm retry ad rcv
-  | concurrent    = resolveConcurrent nss        gens        q edns tm retry ad rcv
-  | otherwise     = resolveSequential nss        gens        q edns tm retry ad rcv
+  | typ == AXFR   = return $ Left InvalidAXFRLookup
+  | onlyOne       = resolveOne        (head nss) (head gens) q tm retry ctls rcv
+  | concurrent    = resolveConcurrent nss        gens        q tm retry ctls rcv
+  | otherwise     = resolveSequential nss        gens        q tm retry ctls rcv
   where
     q = case BS.last dom of
-          '.' -> [Question dom typ]
-          _   -> [Question (dom <> ".") typ]
+          '.' -> Question dom typ
+          _   -> Question (dom <> ".") typ
 
     gens = NE.toList $ genIds rlv
 
     seed    = resolvseed rlv
     nss     = NE.toList $ nameservers seed
     onlyOne = length nss == 1
+    ctls    = qctls <> resolvQueryControls (resolvconf $ resolvseed rlv)
 
     conf       = resolvconf seed
     concurrent = resolvConcurrent conf
     tm         = resolvTimeout conf
     retry      = resolvRetry conf
-    edns       = resolvEDNS conf
+
 
 resolveSequential :: [AddrInfo] -> [IO Identifier] -> Rslv1
-resolveSequential nss gs q edns tm retry ad rcv = loop nss gs
+resolveSequential nss gs q tm retry ctls rcv = loop nss gs
   where
-    loop [ai]     [gen] = resolveOne ai gen q edns tm retry ad rcv
+    loop [ai]     [gen] = resolveOne ai gen q tm retry ctls rcv
     loop (ai:ais) (gen:gens) = do
-        eres <- resolveOne ai gen q edns tm retry ad rcv
+        eres <- resolveOne ai gen q tm retry ctls rcv
         case eres of
           Left  _ -> loop ais gens
           res     -> return res
     loop _  _     = error "resolveSequential:loop"
 
 resolveConcurrent :: [AddrInfo] -> [IO Identifier] -> Rslv1
-resolveConcurrent nss gens q edns tm retry ad rcv = do
+resolveConcurrent nss gens q tm retry ctls rcv = do
     asyncs <- mapM mkAsync $ zip nss gens
     snd <$> waitAnyCancel asyncs
   where
-    mkAsync (ai,gen) = async $ resolveOne ai gen q edns tm retry ad rcv
+    mkAsync (ai,gen) = async $ resolveOne ai gen q tm retry ctls rcv
 
 resolveOne :: AddrInfo -> IO Identifier -> Rslv1
-resolveOne ai gen q edns tm retry ad rcv = do
-    ident <- gen
-    E.try $ udpTcpLookup edns retry rcv ident ai q tm ad
+resolveOne ai gen q tm retry ctls rcv =
+    E.try $ udpTcpLookup gen retry rcv ai q tm ctls
 
 ----------------------------------------------------------------
 
-udpTcpLookup :: UdpRslv
-udpTcpLookup edns retry rcv ident ai q tm ad =
-    udpLookup edns retry rcv ident ai q tm ad `E.catch` \TCPFallback ->
-        tcpLookup ident ai q tm ad
+-- UDP attempts must use the same ID and accept delayed answers
+-- but we use a fresh ID for each TCP lookup.
+--
+udpTcpLookup :: IO Identifier -> UdpRslv
+udpTcpLookup gen retry rcv ai q tm ctls = do
+    ident <- gen
+    udpLookup ident retry rcv ai q tm ctls `E.catch`
+            \TCPFallback -> tcpLookup gen ai q tm ctls
 
 ----------------------------------------------------------------
 
 ioErrorToDNSError :: AddrInfo -> String -> IOError -> IO DNSMessage
-ioErrorToDNSError ai tag ioe = throwIO $ NetworkFailure aioe
+ioErrorToDNSError ai protoName ioe = throwIO $ NetworkFailure aioe
   where
-    aioe = annotateIOError ioe (show ai) Nothing $ Just tag
+    loc = protoName ++ "@" ++ show (addrAddress ai)
+    aioe = annotateIOError ioe loc Nothing Nothing
 
 ----------------------------------------------------------------
 
@@ -129,30 +153,29 @@ udpOpen ai = do
     return sock
 
 -- This throws DNSError or TCPFallback.
-udpLookup :: UdpRslv
-udpLookup edns retry rcv ident ai q tm ad = do
-    let qry = encodeQuestions ident q edns ad
-        ednsRetry = not $ null edns
-    E.handle (ioErrorToDNSError ai "UDP") $
-      bracket (udpOpen ai) close (loop qry ednsRetry 0 RetryLimitExceeded)
+udpLookup :: Identifier -> UdpRslv
+udpLookup ident retry rcv ai q tm ctls = do
+    let qry = encodeQuestion ident q ctls
+    E.handle (ioErrorToDNSError ai "udp") $
+      bracket (udpOpen ai) close (loop qry ctls 0 RetryLimitExceeded)
   where
-    loop qry ednsRetry cnt err sock
+    loop qry lctls cnt err sock
       | cnt == retry = E.throwIO err
       | otherwise    = do
           mres <- timeout tm (send sock qry >> getAns sock)
           case mres of
-              Nothing  -> loop qry ednsRetry (cnt + 1) RetryLimitExceeded sock
+              Nothing  -> loop qry lctls (cnt + 1) RetryLimitExceeded sock
               Just res -> do
-                      let flgs = flags$ header res
-                          truncated = trunCation flgs
-                          rc = rcode flgs
-                      if truncated then
-                          E.throwIO TCPFallback
-                      else if ednsRetry && rc == FormatErr then
-                          let nonednsQuery = encodeQuestions ident q [] ad
-                          in loop nonednsQuery False cnt RetryLimitExceeded sock
-                      else
-                          return res
+                      let fl = flags $ header res
+                          tc = trunCation fl
+                          rc = rcode fl
+                          eh = ednsHeader res
+                          cs = ednsEnabled FlagClear <> lctls
+                      if tc then E.throwIO TCPFallback
+                      else if rc == FormatErr && eh == NoEDNS && cs /= lctls
+                      then let qry' = encodeQuestion ident q cs
+                            in loop qry' cs cnt RetryLimitExceeded sock
+                      else return res
 
     -- | Closed UDP ports are occasionally re-used for a new query, with
     -- the nameserver returning an unexpected answer to the wrong socket.
@@ -162,9 +185,9 @@ udpLookup edns retry rcv ident ai q tm ad = do
     -- instead we'll time out if no matching answer arrives.
     --
     getAns sock = do
-        mres <- rcv sock
-        if checkResp q ident mres
-        then return mres
+        resp <- rcv sock
+        if checkResp q ident resp
+        then return resp
         else getAns sock
 
 ----------------------------------------------------------------
@@ -179,22 +202,29 @@ tcpOpen peer = case peer of
 -- Perform a DNS query over TCP, if we were successful in creating
 -- the TCP socket.
 -- This throws DNSError only.
-tcpLookup :: TcpRslv
-tcpLookup ident ai q tm ad =
-    E.handle (ioErrorToDNSError ai "TCP") $ bracket (tcpOpen addr) close perform
+tcpLookup :: IO Identifier -> TcpRslv
+tcpLookup gen ai q tm ctls =
+    E.handle (ioErrorToDNSError ai "tcp") $ do
+        res <- bracket (tcpOpen addr) close (perform ctls)
+        let rc = rcode $ flags $ header res
+            eh = ednsHeader res
+            cs = ednsEnabled FlagClear <> ctls
+        -- If we first tried with EDNS, retry without on FormatErr.
+        if rc == FormatErr && eh == NoEDNS && cs /= ctls
+        then bracket (tcpOpen addr) close (perform cs)
+        else return res
   where
     addr = addrAddress ai
-    perform vc = do
-        let qry = encodeQuestions ident q [] ad
+    perform cs vc = do
+        ident <- gen
+        let qry = encodeQuestion ident q cs
         mres <- timeout tm $ do
             connect vc addr
             sendVC vc qry
             receiveVC vc
         case mres of
-            Nothing                     -> E.throwIO TimeoutExpired
-            Just res
-                | checkResp q ident res -> return res
-                | otherwise             -> E.throwIO SequenceNumberMismatch
+            Nothing  -> E.throwIO TimeoutExpired
+            Just res -> maybe (return res) E.throwIO (checkRespM q ident res)
 
 ----------------------------------------------------------------
 
